@@ -20,6 +20,7 @@
 
 #include "fonts-pangocairo.hh"
 
+#include "cairo-glue.hh"
 #include "debug.h"
 #include "vtedefines.hh"
 
@@ -238,7 +239,12 @@ FontInfo::FontInfo(vte::glib::RefPtr<PangoContext> context)
 		auto const width = PANGO_PIXELS_CEIL(pango_font_metrics_get_approximate_char_width(metrics.get()));
 #endif /* 0 */
 
-                if (ascent > 0 && height > 0) {
+                /* Sometimes, the metrics return a lower height than the one we measured
+                 * in measure_font(), causing cut-off at the bottom of the last line, see
+                 * https://gitlab.gnome.org/GNOME/gnome-terminal/-/issues/340 . Therefore
+                 * we only use the metrics when its height is at least that which we measured.
+                 */
+                if (ascent > 0 && height >= m_height) {
                         _vte_debug_print(VTE_DEBUG_PANGOCAIRO, "Using pango metrics\n");
 
                         m_ascent = ascent;
@@ -246,6 +252,11 @@ FontInfo::FontInfo(vte::glib::RefPtr<PangoContext> context)
 #if 0
                         m_width = width;
 #endif
+                } else if (ascent >= 0 && height > 0) {
+                        _vte_debug_print(VTE_DEBUG_PANGOCAIRO, "Disregarding pango metrics due to incorrect height (%d < %d)\n",
+                                         height, m_height);
+                } else {
+                        _vte_debug_print(VTE_DEBUG_PANGOCAIRO, "Not using pango metrics due to not providing height or ascent\n");
                 }
 	}
 #endif /* pango >= 1.44 */
@@ -351,17 +362,40 @@ FontInfo::create_for_context(vte::glib::RefPtr<PangoContext> context,
 	if (desc)
 		pango_context_set_font_description(context.get(), desc);
 
-	pango_context_set_language(context.get(), language);
+        if (language != nullptr &&
+            language != pango_context_get_language(context.get()))
+                pango_context_set_language(context.get(), language);
 
-        /* Make sure our contexts have a font_options set.  We use
-          * this invariant in our context hash and equal functions.
-          */
-        if (!pango_cairo_context_get_font_options(context.get())) {
-                cairo_font_options_t *font_options;
+        // Ensure Pango and cairo are configured to quantize and hint font metrics.
+        // Terminal cells in vte have integer pixel sizes. If Pango is configured to do sub-pixel
+        // glyph advances, a small fractional part might get rounded up to a whole pixel - so the
+        // character spacing will appear too wide. Setting the cairo hint metrics option ensures
+        // that there are integer numbers of pixels both above and below the baseline.
+        {
+                auto font_options = vte::take_freeable(cairo_font_options_create());
+#if VTE_GTK == 4
+                cairo_font_options_set_hint_metrics(font_options.get(),
+                                                    CAIRO_HINT_METRICS_ON);
+#endif /* VTE_GTK == 4 */
 
-                font_options = cairo_font_options_create ();
-                pango_cairo_context_set_font_options(context.get(), font_options);
-                cairo_font_options_destroy (font_options);
+                if (auto const ctx_font_options =
+                    pango_cairo_context_get_font_options(context.get())) {
+                        auto const merged_font_options =
+                                vte::take_freeable(cairo_font_options_copy(ctx_font_options));
+                        cairo_font_options_merge(merged_font_options.get(),
+                                                 font_options.get());
+                        pango_cairo_context_set_font_options(context.get(),
+                                                             merged_font_options.get());
+                } else {
+                        // Make sure our contexts have a font_options set.  We use
+                        // this invariant in our context hash and equal functions.
+                        pango_cairo_context_set_font_options(context.get(),
+                                                             font_options.get());
+                }
+
+#if VTE_GTK == 4
+                pango_context_set_round_glyph_positions (context.get(), true);
+#endif /* VTE_GTK == 4 */
         }
 
 	if (G_UNLIKELY(s_font_info_for_context == nullptr))
@@ -398,19 +432,16 @@ FontInfo*
 FontInfo::create_for_widget(GtkWidget* widget,
                             PangoFontDescription const* desc)
 {
-        auto context = gtk_widget_get_pango_context(widget);
-        auto language = pango_context_get_language(context);
-
 #if VTE_GTK == 3
 	auto screen = gtk_widget_get_screen(widget);
-	return create_for_screen(screen, desc, language);
+	return create_for_screen(screen, desc, nullptr);
 #elif VTE_GTK == 4
         auto display = gtk_widget_get_display(widget);
         auto settings = gtk_settings_get_for_display(display);
         auto fontconfig_timestamp = guint{};
         g_object_get (settings, "gtk-fontconfig-timestamp", &fontconfig_timestamp, nullptr);
-        return create_for_context(vte::glib::make_ref(context),
-                                  desc, language, fontconfig_timestamp);
+        return create_for_context(vte::glib::take_ref(gtk_widget_create_pango_context(widget)),
+                                  desc, nullptr, fontconfig_timestamp);
         // FIXMEgtk4: this uses a per-widget context, while the gtk3 code uses a per-screen
         // one. That means there may be a lot less sharing and a lot more FontInfo's around?
 #endif
