@@ -92,6 +92,7 @@ class NamedMode:
     number: int
     name: str
     default: bool
+    preserve_decstr: bool=False
     flags: Flags=Flags.NONE
     source: typing.Optional[Source]=None
     alias: typing.Optional[typing.List[str]]=None
@@ -226,7 +227,7 @@ modes = [
     # References: ECMA-48 § F.5.2 Removed!
     #             VT525
     #
-    mode_ECMA('LNM', 20, default=False),
+    mode_ECMA('LNM', 20, default=False, preserve_decstr=True),
 
     mode_ECMA('GRCM', 21, default=True),
 
@@ -369,14 +370,14 @@ modes = [
     # Sets page width to 132 (set) or 80 (reset) columns.
     #
     # Changing this mode resets the top, bottom, left, right margins;
-    # clears the screen (unless DECNCSM is set); resets DECLRMM; and clears
+    # clears the screen (unless DECNCSM is set); resets DECLRMM; and clearsb
     # the status line if host-writable.
     #
     # Default: reset
     #
     # References: VT525
     #
-    mode_WHAT('DEC_132_COLUMN', 3, default=False, flags=Flags.WRITABLE),
+    mode_WHAT('DEC_132_COLUMN', 3, default=False, preserve_decstr=True, flags=Flags.WRITABLE),
 
     # DECANM - ansi-mode
     # Resetting this puts the terminal into VT52 compatibility mode.
@@ -407,7 +408,7 @@ modes = [
     #
     # References: VT525
     #
-    mode_WHAT('DEC_REVERSE_IMAGE', 5, default=False, flags=Flags.WRITABLE),
+    mode_WHAT('DEC_REVERSE_IMAGE', 5, default=False, preserve_decstr=True, flags=Flags.WRITABLE),
 
     # DECOM - origin mode
     # If set, the cursor is restricted to within the page margins.
@@ -442,7 +443,7 @@ modes = [
     #
     # Probably not worth implementing.
     #
-    mode_WHAT('DECARM', 8, default=True),
+    mode_WHAT('DECARM', 8, default=True, preserve_decstr=True),
 
     mode_WHAT('XTERM_MOUSE_X10', 9, default=False, flags=Flags.WRITABLE),
     mode_WHAT('DECLTM', 11, default=False),
@@ -543,6 +544,7 @@ modes = [
     #mode_WHAT('DECGPCM', 44, default=False),
 
     # DECGCPS - Graphics Print Colour Syntax
+    # If set, uses RGB colour format; if reset, uses HLS colour format.
     #
     # Default: reset
     #
@@ -707,14 +709,16 @@ modes = [
     mode_WHAT('DECXRLM', 73, default=False),
 
     # DECSDM - sixel display mode (scrolling)
+    # If set, SIXEL scrolling is disabled; when reset, SIXEL scrolling
+    # is enabled.
     #
-    # Default: set
+    # Default: reset
     #
     # References: ?
     #
     # Note: Conflicts with WY161
     #
-    mode_WHAT('DECSDM', 80, default=True, flags=Flags.WRITABLE),
+    mode_WHAT('DECSDM', 80, default=False, flags=Flags.WRITABLE),
 
     # DECKPM - key position mode
     # If set, the keyboard sends extended reports (DECEKBD) that include
